using System;
using System.Collections;
using System.Data;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using CMS.GlobalHelper;
using CMS.SiteProvider;
using CMS.DataEngine;
using CMS.EventLog;
using CMS.CMSHelper;
using CMS.UIControls;

public partial class CMSSiteManager_Administration_EventLog_EventLog : CMSEventLogPage, ICallbackEventHandler
{
    #region "Protectetd variables"

    protected int siteId = -1;
    protected int eventId = 0;
    protected EventLogProvider eventProvider = new EventLogProvider();
    protected Hashtable mParameters = null;

    #endregion


    #region "Private properties"

    /// <summary>
    /// Dialog control identificator
    /// </summary>
    private string Identificator
    {
        get
        {
            string identificator = hdnIdentificator.Value;
            if (string.IsNullOrEmpty(identificator))
            {
                identificator = Guid.NewGuid().ToString();
                hdnIdentificator.Value = identificator;
            }

            return identificator;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Register the dialog script
        ScriptHelper.RegisterClientScriptBlock(this, typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

        string openEventDetailScript = "var dialogParams = '';\n" +
                    "function OpenEventDetail(queryParameters) {\n" +
                    "modalDialog(" + ScriptHelper.GetString(ResolveUrl("~/CMSSiteManager/Administration/EventLog/EventLog_Details.aspx")) + " + queryParameters, 'eventdetails', 920, 700);\n" +
                    "}";

        // Register the dialog script
        ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "EventLog_OpenDetail", ScriptHelper.GetScript(openEventDetailScript));

        // Setup page title text and image
        CurrentMaster.Title.TitleText = ResHelper.GetString("EventLogList.Header");
        CurrentMaster.Title.TitleImage = GetImageUrl("Objects/CMS_EventLog/object.png");

        CurrentMaster.Title.HelpTopicName = "list_of_events";
        CurrentMaster.Title.HelpName = "helpTopic";

        lblSite.Text = ResHelper.GetString("EventLogList.Label");

        gridEvents.OnDataReload += new UniGrid.OnDataReloadEventHandler(gridEvents_OnDataReload);
        gridEvents.GridView.RowDataBound += GridView_RowDataBound;
        gridEvents.OnExternalDataBound += new OnExternalDataBoundEventHandler(gridEvents_OnExternalDataBound);

        if (!RequestHelper.IsPostBack())
        {
            gridEvents.OrderBy = "EventTime DESC, EventID DESC";
        }

        btnClearLog.Text = ResHelper.GetString("EventLogList.Clear");
        btnClearLog.Attributes.Add("onclick", "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("EventLogList.ClearAllConfirmation")) + ");");

        siteId = QueryHelper.GetInteger("siteid", 0);
        if (siteId > 0)
        {
            pnlSites.Visible = false;
        }
        else
        {
            // Set site selector
            siteSelector.DropDownSingleSelect.AutoPostBack = true;
            siteSelector.OnlyRunningSites = false;
            siteSelector.AllowAll = true;
            siteSelector.UniSelector.SpecialFields = new string[1, 2] { { ResHelper.GetString("EventLogList.GlobalEvents"), "0" } };
            siteSelector.UniSelector.OnSelectionChanged += UniSelector_OnSelectionChanged;

            if (!RequestHelper.IsPostBack())
            {
                // Preselect (all events)
                siteId = -1;
            }
            else
            {
                siteId = ValidationHelper.GetInteger(siteSelector.Value, 0);
            }
        }

        cntFilter.SiteID = siteId;
        gridEvents.WhereCondition = cntFilter.WhereCondition;
    }


    protected void Page_PreRender(object sender, EventArgs e)
    {
        // If events grid not empty allow clear log button
        btnClearLog.Enabled = (gridEvents.GridView.Rows.Count != 0);
        if (btnClearLog.Enabled)
        {
            btnClearLog.Attributes.Add("onclick", "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("EventLogList.ClearConfirmation")) + ");");
        }
    }

    #endregion


    #region "Control methods"

    /// <summary>
    /// Handles site selection change event
    /// </summary>
    protected void UniSelector_OnSelectionChanged(object sender, EventArgs e)
    {
        // Update grid
        pnlUpdate.Update();
    }


    /// <summary>
    /// Deletes event logs from DB.
    /// </summary>
    protected void ClearLogButton_Click(object sender, EventArgs e)
    {
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.EventLog", "ClearLog"))
        {
            RedirectToAccessDenied("CMS.EventLog", "ClearLog");
        }

        GeneralConnection genConn = ConnectionHelper.GetConnection();
        EventLogProvider eventProvider = new EventLogProvider(genConn);
        UserInfo ui = CMSContext.CurrentUser;

        // Deletes event logs of specific site from DB
        eventProvider.ClearEventLog(ui.UserID, ui.UserName, Request.UserHostAddress, siteId);

        gridEvents.ReloadData();
    }


    protected void GridView_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            string color = null;
            string code = ValidationHelper.GetString(((DataRowView)(e.Row.DataItem)).Row["EventType"], string.Empty);
            switch (code.ToLower())
            {
                case "e":
                    if ((e.Row.RowIndex & 1) == 1)
                    {
                        color = "#EEC9C9";
                    }
                    else
                    {
                        color = "#FFDADA";
                    }
                    break;

                case "w":
                    if ((e.Row.RowIndex & 1) == 1)
                    {
                        color = "#EEEEC9";
                    }
                    else
                    {
                        color = "#FFFFDA";
                    }
                    break;
            }

            if (!string.IsNullOrEmpty(color))
            {
                e.Row.Style.Add("background-color", color);
            }
        }
    }


    protected DataSet gridEvents_OnDataReload(string completeWhere, string currentOrder, int currentTopN, int currentOffset, int currentPageSize, ref int totalRecords)
    {
        string columns = "EventID,EventType,EventTime,Source,EventCode,UserName,IPAddress,DocumentName,SiteDisplayName,EventMachineName,ROW_NUMBER() OVER (ORDER BY " + currentOrder + ") AS RowNumber";
        return eventProvider.GetAllEvents(completeWhere, currentOrder, 0, columns, currentOffset, currentPageSize, ref totalRecords);
    }


    protected object gridEvents_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        if (sourceName == "eventtype")
        {
            string evetType = ValidationHelper.GetString(parameter, "");
            return "<div style=\"width:100%;text-align:center;cursor:help;\" title=\"" + HTMLHelper.HTMLEncode(EventLogHelper.GetEventTypeText(evetType)) + "\">" + evetType + " </div>";
        }
        else if (sourceName == "formattedusername")
        {
            return HTMLHelper.HTMLEncode(Functions.GetFormattedUserName(Convert.ToString(parameter)));
        }
        else if (sourceName == "edit")
        {
            string caption = ResHelper.GetString("Unigrid.EventLog.Actions.Display");
            string imageUrl = GetImageUrl("Design/Controls/UniGrid/Actions/View.png");
            string parameters = Convert.ToString(parameter);

            if (siteId > 0)
            {
                parameters += "|" + siteId.ToString();
            }

            string action = "dialogParams = '" + parameters + "';" + ClientScript.GetCallbackEventReference(this, "dialogParams", "OpenEventDetail", null) + ";return false;";

            return "<input type=\"image\" style=\"border-width: 0px; margin: 0px 3px;\" alt=\"" + caption +
                "\" src=\"" + imageUrl + "\" class=\"UnigridActionButton\" title=\"" + caption + "\" onclick=\"" + action + "\">";
        }

        return parameter;
    }

    #endregion


    #region "ICallbackEventHandler Members"

    /// <summary>
    /// Get callback result
    /// </summary>
    public string GetCallbackResult()
    {
        string orderby = this.gridEvents.SortDirect;
        string whereCondition = this.gridEvents.WhereCondition;

        mParameters = new Hashtable();
        mParameters["where"] = whereCondition;
        mParameters["orderby"] = this.gridEvents.SortDirect;

        WindowHelper.Add(Identificator, mParameters);

        string queryString = "?params=" + Identificator;

        if (siteId > 0)
        {
            queryString = UrlHelper.AddParameterToUrl(queryString, "siteid", siteId.ToString());
        }
        queryString = UrlHelper.AddParameterToUrl(queryString, "hash", QueryHelper.GetHash(queryString));
        queryString = UrlHelper.AddParameterToUrl(queryString, "eventid", eventId.ToString());

        return queryString;
    }


    /// <summary>
    /// Raise callback method
    /// </summary>
    public void RaiseCallbackEvent(string eventArgument)
    {
        string[] parameters = eventArgument.Split('|');
        if ((parameters != null) && (parameters.Length > 0))
        {
            eventId = ValidationHelper.GetInteger(parameters[0], 0);
            if (parameters.Length > 1)
            {
                siteId = ValidationHelper.GetInteger(parameters[1], 0);
            }
        }
    }

    #endregion
}

